<?php
namespace Bricks;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Element_Template extends Element {
	public $block    = 'core/template';
	public $category = 'general';
	public $name     = 'template';
	public $icon     = 'ti-layers';

	public function get_label() {
		return esc_html__( 'Template', 'bricks' );
	}

	public function set_controls() {
		$this->controls['template'] = [
			'tab'         => 'content',
			'label'       => esc_html__( 'Template', 'bricks' ),
			'type'        => 'select',
			'options'     => bricks_is_builder() ? Templates::get_templates_list( get_the_ID() ) : [],
			'searchable'  => true,
			'placeholder' => esc_html__( 'Select template', 'bricks' ),
		];
	}

	public function render() {
		$settings    = $this->settings;
		$template_id = ! empty( $settings['template'] ) ? intval( $settings['template'] ) : false;

		if ( ! $template_id ) {
			return $this->render_element_placeholder(
				[
					'title' => esc_html__( 'No template selected.', 'bricks' ),
				]
			);
		}

		if ( $template_id == $this->post_id || $template_id == get_the_ID() ) {
			return $this->render_element_placeholder(
				[
					'title' => esc_html__( 'Not allowed: Infinite template loop.', 'bricks' ),
				]
			);
		}

		$template_data = get_post_meta( $template_id, BRICKS_DB_PAGE_CONTENT, true );

		if ( empty( $template_data ) || ! is_array( $template_data ) ) {
			return $this->render_element_placeholder(
				[
					'title' => esc_html__( 'Your selected template is empty.', 'bricks' ),
				]
			);
		}

		// Apply the property "_noRootClass" to avoid adding the "root" class when rendering templates @see: base.php::set_root_attributes()
		// foreach ( $template_data as $key => $element ) {
		// if ( empty( $element['parent'] ) ) {
		// $template_data[ $key ]['_noRootClass'] = true;
		// }
		// }

		// Avoid infinite loop
		static $rendered_templates = [];

		if ( ! array_key_exists( $template_id, $rendered_templates ) ) {

			// Store template to avoid loops
			$rendered_templates[ $template_id ] = 1;

			// Store the current main render_data self::$elements
			$store_elements = Frontend::$elements;

			echo "<div {$this->render_attributes( '_root' )}>";
			echo Frontend::render_data( $template_data );
			echo '</div>';

			// Reset the main render_data self::$elements
			Frontend::$elements = $store_elements;

			// Remove the template from our loop control
			unset( $rendered_templates[ $template_id ] );
		}
	}

	public static function render_builder() { ?>
		<script type="text/x-template" id="tmpl-bricks-element-template">
			<component
				:is="'template-static'"
				v-if="settings.template"
				:templateId="settings.template">
			</component>

			<div v-else v-html="renderElementPlaceholder()"></div>
		</script>
		<?php
	}
}
